package com.galaxis.manatee.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.ChuanyunSelfCostDao;
import com.galaxis.manatee.dao.ChuanyunSelfWorkHourDao;
import com.galaxis.manatee.dao.ChuanyunWeekMemberHourDao;
import com.galaxis.manatee.dao.ChuanyunWorkHourDao;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunFindAllBizDTO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunFindAllBizReturnData;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.util.TimeUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.util.*;

/**
 * @author zcj
 * @version 0.1
 * @date 2021/3/4 4:27 上午
 */
@Slf4j
@Service
public class LogCheckService {
    private final ObjectMapper objectMapper = new ObjectMapper();
    private final ChuanyunWorkHourDao chuanyunWorkHourDao;
    private final ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunWeekMemberHourDao chuanyunWeekMemberHourDao;
    private final ChuanyunSelfCostDao chuanyunSelfCostDao;

    public LogCheckService(ChuanyunWorkHourDao chuanyunWorkHourDao, ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao, ChuanYunManager chuanYunManager, ChuanyunWeekMemberHourDao chuanyunWeekMemberHourDao, ChuanyunSelfCostDao chuanyunSelfCostDao) {
        this.chuanyunWorkHourDao = chuanyunWorkHourDao;
        this.chuanyunSelfWorkHourDao = chuanyunSelfWorkHourDao;
        this.chuanYunManager = chuanYunManager;
        this.chuanyunWeekMemberHourDao = chuanyunWeekMemberHourDao;
        this.chuanyunSelfCostDao = chuanyunSelfCostDao;
    }

    /**
     * 根据当前userId核对以下内容：
     * 日志原始数据每日各个项目工时信息（可使用ChuanyunWorkHourDao中方法进行查询），与系统中保存每日工时（ChuanyunSelfWorkHourDO）进行比较
     * 完成核对后，删除数据库中多余的每周工时
     * 完成核对后，删除氚云中中多余的每周工时
     *
     * @param userId 用户Id
     * @throws Exception 用户Id为空异常
     */
    public void checkDayWorkHour(String userId) throws Exception {
        if (StringUtils.isEmpty(userId)) {
            throw new Exception("用户Id为空");
        }
        long start = System.currentTimeMillis();
        log.info("开始核对日志原始工时与标准化工时");
        //根据日志原始数据获取单个人的所有工时记录
        List<ChuanyunWorkHourDO> chuanyunWorkHourList = chuanyunWorkHourDao.getSelfWorkHour(userId);
        //根据标准化工时获取单个人的标准化工时记录
        List<ChuanyunSelfWorkHourDO> chuanyunSelfWorkHourList = chuanyunSelfWorkHourDao.findByUserId(userId);
        //删除多余每日工时
        compareDayWorkHour(chuanyunWorkHourList, chuanyunSelfWorkHourList);
        log.info("核对耗时" + (System.currentTimeMillis() - start) + "毫秒");
    }

    /**
     * 根据当前userId核对以下内容：
     * 标准化每日工时统计每周工时的结果（可使用ChuanyunWeekMemberHourDao中方法进行查询），与系统中保存每周工时(ChuanyunWeekMemberHourDO)进行比较
     * 完成核对后，更新或删除数据库中多余的每周工时
     * 完成核对后，更新或删除氚云中中多余的每周工时
     *
     * @param userId 用户Id
     * @throws Exception 用户Id为空异常
     */
    public void checkWeekWorkHour(String userId) throws Exception {
        if (StringUtils.isEmpty(userId)) {
            throw new Exception("用户Id为空");
        }
        long start = System.currentTimeMillis();
        log.info("开始核对标准化周工时");
        // 根据日工时聚合得到周工时
        List<ChuanyunWeekMemberHourDO> list = chuanyunWeekMemberHourDao.getWeekMemberHourByUserId(userId);
        // 数据库已存在的周工时
        List<ChuanyunWeekMemberHourDO> oldList = chuanyunWeekMemberHourDao.findByUserId(userId);
        Map<String, Integer> map = new HashMap<>(list.size() + oldList.size());

        if (list.size() != oldList.size()) {
            for (ChuanyunWeekMemberHourDO chuanyunWeekMemberHourDO : list) {
                String key = chuanyunWeekMemberHourDO.getUserId() + chuanyunWeekMemberHourDO.getProjectId() + chuanyunWeekMemberHourDO.getProjectType() + chuanyunWeekMemberHourDO.getYear() + chuanyunWeekMemberHourDO.getWeek();
                map.put(key, 1);
            }
            for (ChuanyunWeekMemberHourDO chuanyunWeekMemberHourDO : oldList) {
                String key = chuanyunWeekMemberHourDO.getUserId() + chuanyunWeekMemberHourDO.getProjectId() + chuanyunWeekMemberHourDO.getProjectType() + chuanyunWeekMemberHourDO.getYear() + chuanyunWeekMemberHourDO.getWeek();
                Integer count = map.get(key);
                if (count == null) {
                    chuanyunWeekMemberHourDao.delete(chuanyunWeekMemberHourDO);
                    List<String> matchers = new ArrayList<>();
                    matchers.add("F0000001_2," + chuanyunWeekMemberHourDO.getProjectId());
                    matchers.add("F0000002_2," + chuanyunWeekMemberHourDO.getUserId());
                    matchers.add("F0000006_2," + chuanyunWeekMemberHourDO.getWeek());
                    matchers.add("F0000008_2," + chuanyunWeekMemberHourDO.getYear());

                    var filter = Filter.instance(0, 1, true, "And", matchers);
                    var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunProjectMemberWeekHourDO.SCHEMA_CODE, filter);
                    ChuanyunSaveDTO chuanyunSaveDTO;
                    if (chuanyunFindAllResponse.getReturnData() != null) {
                        List<ChuanyunProjectMemberWeekHourDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                        });
                        chuanyunSaveDTO = chuanYunManager.delete(ChuanyunProjectMemberWeekHourDO.SCHEMA_CODE, result.get(0).getObjectId());
                    }
                    log.info(chuanyunWeekMemberHourDO.getUserId() + "删除项目" + chuanyunWeekMemberHourDO.getProjectId() + "时间为" + chuanyunWeekMemberHourDO.getYear() + "年" + chuanyunWeekMemberHourDO.getWeek() + "周工时");
                }
            }
        }
        log.info("核对耗时" + (System.currentTimeMillis() - start) + "毫秒");
    }

    /**
     * 根据当前userId核对以下内容：
     * 标准化每日工时统计每月工时的结果（可用ChuanyunSelfWorkHourDao中的方法），与系统中保存每月工时(ChuanyunSelfCostDO)进行比较
     * 完成核对后，更新或删除数据库中多余的每周工时
     * 完成核对后，更新或删除氚云中中多余的每周工时
     *
     * @param userId 用户Id
     * @throws Exception 用户Id为空异常
     */
    public void checkMonthWorkHour(String userId) throws Exception {
        if (StringUtils.isEmpty(userId)) {
            throw new Exception("用户Id为空");
        }

        long start = System.currentTimeMillis();
        log.info("开始核对标准化月工时");
        List<ChuanyunSelfWorkHourDO> list = chuanyunSelfWorkHourDao.getMonthProjectHour(userId);
        List<ChuanyunSelfCostDO> costDOList = chuanyunSelfCostDao.findByUserId(userId);
        if (list.size() != costDOList.size()) {
            Map<String, Integer> map = new HashMap<>(list.size() + costDOList.size());
            for (ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO : list) {
                map.put(chuanyunSelfWorkHourDO.getUserId() + chuanyunSelfWorkHourDO.getProjectId() + chuanyunSelfWorkHourDO.getProjectType() + chuanyunSelfWorkHourDO.getDayLogDate().withDayOfMonth(1), 1);
            }
            for (ChuanyunSelfCostDO chuanyunSelfCostDO : costDOList) {
                Integer count = map.get(chuanyunSelfCostDO.getUserId() + chuanyunSelfCostDO.getProjectId() + chuanyunSelfCostDO.getProjectType() + chuanyunSelfCostDO.getDayLogDate().withDayOfMonth(1));
                if (count == null) {
                    chuanyunSelfCostDao.delete(chuanyunSelfCostDO);
                    List<String> matchers = new ArrayList<>();
                    matchers.add("F0000003_2," + chuanyunSelfCostDO.getProjectId());
                    matchers.add("F0000002_2," + chuanyunSelfCostDO.getUserId());
                    matchers.add("F0000004_2," + chuanyunSelfCostDO.getDayLogDate().plusDays(1).withDayOfMonth(1));

                    var filter = Filter.instance(0, 1, true, "And", matchers);
                    var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunMemberMonthCostDO.SCHEMA_CODE, filter);
                    ChuanyunSaveDTO chuanyunSaveDTO;
                    if (chuanyunFindAllResponse.getReturnData() != null) {
                        List<ChuanyunMemberMonthCostDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                        });
                        chuanyunSaveDTO = chuanYunManager.delete(ChuanyunMemberMonthCostDO.SCHEMA_CODE, result.get(0).getObjectId());
                    }
                    log.info(chuanyunSelfCostDO.getUserId() + "删除项目" + chuanyunSelfCostDO.getProjectId() + "时间为" + chuanyunSelfCostDO.getDayLogDate() + "月工时");
                }
            }
        }
        log.info("核对耗时" + (System.currentTimeMillis() - start) + "毫秒");
    }

    /**
     * 删除多余每日工时
     *
     * @param chuanyunWorkHourList     日志原始工时
     * @param chuanyunSelfWorkHourList 标准化工时列表
     */
    private void compareDayWorkHour(List<ChuanyunWorkHourDO> chuanyunWorkHourList, List<ChuanyunSelfWorkHourDO> chuanyunSelfWorkHourList) {
        Map<String, Integer> map = new HashMap<>(64);
        //判断人员项目每日工时数量书否有变动
        if (chuanyunWorkHourList.size() != chuanyunSelfWorkHourList.size()) {
            //如果有变动将
            for (ChuanyunWorkHourDO chuanyunWorkHourDO : chuanyunWorkHourList) {
                String key = chuanyunWorkHourDO.getUserId() + chuanyunWorkHourDO.getProjectId() + chuanyunWorkHourDO.getProjectType() + chuanyunWorkHourDO.getDayLogDate();
                if (map.containsKey(key)) {
                    log.info("重复的key: {}", key);
                }
                map.put(key, 1);
            }
            //比较是否有撤回工时
            for (ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO : chuanyunSelfWorkHourList) {
//                if (!chuanyunSelfWorkHourDO.getProjectType().equals("生产项目")){
                String projectId;
                if (chuanyunSelfWorkHourDO.getProjectId().equals(ChuanYunConstant.PRODUCTION_PROJECT_ID)) {
                    projectId = "";
                } else {
                    projectId = chuanyunSelfWorkHourDO.getProjectId();
                }
                Integer count = map.get(chuanyunSelfWorkHourDO.getUserId() + projectId + chuanyunSelfWorkHourDO.getProjectType() + chuanyunSelfWorkHourDO.getDayLogDate());
                if (count == null) {
                    try {
                        //删除数据库日工时
                        chuanyunSelfWorkHourDao.delete(chuanyunSelfWorkHourDO);
                        //删除氚云日工时
                        List<String> matchers = new ArrayList<>();
                        if (!StringUtils.isEmpty(chuanyunSelfWorkHourDO.getProjectId()) && !chuanyunSelfWorkHourDO.getProjectId().equals(ChuanYunConstant.PRODUCTION_PROJECT_ID)) {
                            matchers.add("F0000001_2," + chuanyunSelfWorkHourDO.getProjectId());
                        }
                        matchers.add("F0000002_2," + chuanyunSelfWorkHourDO.getUserId());
                        matchers.add("F0000003_2," + TimeUtil.formatDateTime(chuanyunSelfWorkHourDO.getDayLogDate()));

                        var filter = Filter.instance(-1, 100, true, "And", matchers);
                        var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunMemberHourDO.SCHEMA_CODE, filter);
                        // 修改bug 删除氚云数据库不生效问题
                        // 1.日期未按照氚云要求格式化。
                        // 2.查询条件未从-1开始查，且只查了1条。
                        // 3.转换json时对象的objectId为null，无法删除氚云数据库。父类中属性需要在转换时指定转换的结果类型
                        if (chuanyunFindAllResponse.getReturnData() != null) {
                            List<Object> responses = chuanyunFindAllResponse.getReturnData().getBizObjectArray();
                            ObjectMapper objectMapper = new ObjectMapper();
                            responses.forEach(o -> {
                                ChuanyunMemberHourDO chuanyunMemberHourDO = objectMapper.convertValue(o, ChuanyunMemberHourDO.class);
                                chuanYunManager.delete(ChuanyunMemberHourDO.SCHEMA_CODE, chuanyunMemberHourDO.getObjectId());
                            });
                        }
                        log.info(chuanyunSelfWorkHourDO.getUserId() + "删除" + chuanyunSelfWorkHourDO.getDayLogDate() + "位于" + chuanyunSelfWorkHourDO.getProjectId() + "工时");
                    } catch (Exception e) {
                        e.printStackTrace();
                        log.error(e.getMessage());
                    }
                }
//                }
            }
        }
    }

    /**
     * 校验指定时间后的日工时记录
     *
     * @param allUserIdList 用户id集合
     * @param afterDay      多少天后
     */
    public void checkDayWorkHourTimeAfter(List<String> allUserIdList, Integer afterDay) {
        allUserIdList.parallelStream().forEach(userId -> {
            if (Objects.isNull(userId)){
                log.error("用户Id为null");
                return;
            }
            List<ChuanyunWorkHourDO> halfYearSelfWorkHour = chuanyunWorkHourDao.getHalfYearSelfWorkHour(userId);
            LocalDateTime localDateTime = LocalDateTime.now().minusDays(afterDay);
            List<ChuanyunSelfWorkHourDO> chuanyunSelfWorkHourList = chuanyunSelfWorkHourDao.findByUserIdAndDayLogDateAfter(userId, localDateTime);
            compareDayWorkHour(halfYearSelfWorkHour, chuanyunSelfWorkHourList);
        });
        log.info("核对日工时记录完成");
    }

    public void checkWeekWorkHour(List<String> allUserIdList) {
        allUserIdList.parallelStream().forEach(userId -> {
            log.info("开始核对用户: {} 标准化周工时", userId);
            if (Objects.isNull(userId)){
                log.error("用户Id为null");
                return;
            }
            // 根据日工时聚合得到周工时
            List<ChuanyunWeekMemberHourDO> list = chuanyunWeekMemberHourDao.getWeekMemberHourByUserId(userId);
            // 数据库已存在的周工时
            List<ChuanyunWeekMemberHourDO> oldList = chuanyunWeekMemberHourDao.findByUserId(userId);
            Map<String, Integer> map = new HashMap<>(list.size() + oldList.size());

            if (list.size() != oldList.size()) {
                for (ChuanyunWeekMemberHourDO chuanyunWeekMemberHourDO : list) {
                    String key = chuanyunWeekMemberHourDO.getUserId() + chuanyunWeekMemberHourDO.getProjectId() + chuanyunWeekMemberHourDO.getProjectType() + chuanyunWeekMemberHourDO.getYear() + chuanyunWeekMemberHourDO.getWeek();
                    map.put(key, 1);
                }
                for (ChuanyunWeekMemberHourDO chuanyunWeekMemberHourDO : oldList) {
                    String key = chuanyunWeekMemberHourDO.getUserId() + chuanyunWeekMemberHourDO.getProjectId() + chuanyunWeekMemberHourDO.getProjectType() + chuanyunWeekMemberHourDO.getYear() + chuanyunWeekMemberHourDO.getWeek();
                    Integer count = map.get(key);
                    if (count == null) {
                        chuanyunWeekMemberHourDao.delete(chuanyunWeekMemberHourDO);
                        List<String> matchers = new ArrayList<>();
                        matchers.add("F0000001_2," + chuanyunWeekMemberHourDO.getProjectId());
                        matchers.add("F0000002_2," + chuanyunWeekMemberHourDO.getUserId());
                        matchers.add("F0000006_2," + chuanyunWeekMemberHourDO.getWeek());
                        matchers.add("F0000008_2," + chuanyunWeekMemberHourDO.getYear());

                        var filter = Filter.instance(0, 1, true, "And", matchers);
                        ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = null;
                        try {
                            chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunProjectMemberWeekHourDO.SCHEMA_CODE, filter);
                        } catch (JsonProcessingException e) {
                            e.printStackTrace();
                        }
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        if (chuanyunFindAllResponse.getReturnData() != null) {
                            List<ChuanyunProjectMemberWeekHourDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                            });
                            chuanyunSaveDTO = chuanYunManager.delete(ChuanyunProjectMemberWeekHourDO.SCHEMA_CODE, result.get(0).getObjectId());
                        }
                        log.info(chuanyunWeekMemberHourDO.getUserId() + "删除项目" + chuanyunWeekMemberHourDO.getProjectId() + "时间为" + chuanyunWeekMemberHourDO.getYear() + "年" + chuanyunWeekMemberHourDO.getWeek() + "周工时");
                    }
                }
            }
        });
    }

    public void checkMonthWorkHour(List<String> allUserIdList) {
        long start = System.currentTimeMillis();
        log.info("开始核对标准化月工时");
        allUserIdList.parallelStream().forEach(userId -> {
            if (Objects.isNull(userId)){
                log.error("用户Id为null");
                return;
            }
            List<ChuanyunSelfWorkHourDO> list = chuanyunSelfWorkHourDao.getMonthProjectHour(userId);
            List<ChuanyunSelfCostDO> costDOList = chuanyunSelfCostDao.findByUserId(userId);
            if (list.size() != costDOList.size()) {
                Map<String, Integer> map = new HashMap<>(list.size() + costDOList.size());
                for (ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO : list) {
                    map.put(chuanyunSelfWorkHourDO.getUserId() + chuanyunSelfWorkHourDO.getProjectId() + chuanyunSelfWorkHourDO.getProjectType() + chuanyunSelfWorkHourDO.getDayLogDate().withDayOfMonth(1), 1);
                }
                for (ChuanyunSelfCostDO chuanyunSelfCostDO : costDOList) {
                    Integer count = map.get(chuanyunSelfCostDO.getUserId() + chuanyunSelfCostDO.getProjectId() + chuanyunSelfCostDO.getProjectType() + chuanyunSelfCostDO.getDayLogDate().withDayOfMonth(1));
                    if (count == null) {
                        chuanyunSelfCostDao.delete(chuanyunSelfCostDO);
                        List<String> matchers = new ArrayList<>();
                        matchers.add("F0000003_2," + chuanyunSelfCostDO.getProjectId());
                        matchers.add("F0000002_2," + chuanyunSelfCostDO.getUserId());
                        matchers.add("F0000004_2," + chuanyunSelfCostDO.getDayLogDate().plusDays(1).withDayOfMonth(1));

                        var filter = Filter.instance(0, 10, true, "And", matchers);
                        ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = null;
                        try {
                            chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunMemberMonthCostDO.SCHEMA_CODE, filter);
                        } catch (JsonProcessingException e) {
                            e.printStackTrace();
                        }
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        if (chuanyunFindAllResponse.getReturnData() != null) {
                            List<ChuanyunMemberMonthCostDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                            });
                            chuanyunSaveDTO = chuanYunManager.delete(ChuanyunMemberMonthCostDO.SCHEMA_CODE, result.get(0).getObjectId());
                        }
                        log.info(chuanyunSelfCostDO.getUserId() + "删除项目" + chuanyunSelfCostDO.getProjectId() + "时间为" + chuanyunSelfCostDO.getDayLogDate() + "月工时");
                    }
                }
            }

        });

        log.info("核对耗时" + (System.currentTimeMillis() - start) + "毫秒");
    }

    /**
     * 按照年月日检验是否存在时区问题的数据
     *
     * @param year   年
     * @param month  月
     * @param maxDay 当月最后一天
     */
    public void checkChuanYunAll(Integer year, Integer month, Integer maxDay) {
        for (int i = 1; i <= maxDay; i++) {
            List<String> matchers = new ArrayList<>();
            LocalDateTime start = LocalDateTime.of(year, month, i, 0, 0, 0);
            LocalDateTime end = LocalDateTime.of(year, month, i, 23, 59, 59);
            String startDateTime = TimeUtil.formatDateTime(start);
            String endDateTime = TimeUtil.formatDateTime(end);
            log.info("startDateTime: {}", startDateTime);
            log.info("endDateTime: {}", endDateTime);
            matchers.add("F0000003_0," + startDateTime);
            matchers.add("F0000003_3," + endDateTime);
            boolean flag = true;
            int startIndex = -1;
            int pageSize = 500;
            while (flag) {
                Filter filter = Filter.instance(startIndex, startIndex + pageSize, true, Filter.AND, matchers);
                ChuanyunFindAllBizDTO<Object> all = null;
                try {
                    all = chuanYunManager.findAll(ChuanyunMemberHourDO.SCHEMA_CODE, filter);
                } catch (JsonProcessingException e) {
                    e.printStackTrace();
                }
                if (all.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (all.getReturnData().getTotalCount() >= startIndex + pageSize) {
                    startIndex += pageSize;
                } else {
                    flag = false;
                }
                ChuanyunFindAllBizReturnData<Object> returnData = all.getReturnData();

                if (returnData != null) {
                    List<Object> bizObjectArray = returnData.getBizObjectArray();
                    ObjectMapper objectMapper = new ObjectMapper();
                    for (Object o : bizObjectArray) {
                        ChuanyunMemberHourDO chuanyunMemberHourDO = objectMapper.convertValue(o, ChuanyunMemberHourDO.class);

                        ChuanyunSaveDTO delete = null;
                        try {
                            delete = chuanYunManager.delete(ChuanyunMemberHourDO.SCHEMA_CODE, chuanyunMemberHourDO.getObjectId());
                        } catch (Exception e) {
                            log.error("删除异常 e: {}", e.getMessage());
                        }
                        if (!delete.getSuccessful()) {
                            log.info("删除失败");
                        }
                        log.info("userId: {}, objectId: {}, time: {}", chuanyunMemberHourDO.getUserId(), chuanyunMemberHourDO.getObjectId(), chuanyunMemberHourDO.getDayLogDate());
                    }
                }
            }

        }
    }
}
